# ============================================================================================================
# Maths.py
# lheywang on 17/12/2024
#
# Base file for the maths class
#
# ============================================================================================================
from ..BaseOptionnalClass import SiglentBase


class SiglentMaths(SiglentBase):
    """
    pySDS [Files][SiglentMaths] :   Class herited from SiglentBase.
                                    Store all command related the control of maths functions.

        Attributes :
            Herited from SiglentBase

        Methods :
            Private (1) :
                __init__ :                  Small overhide to add one variable

            Public (12):
                DefineMathOperation :       Configure math operation
                EnableMathInvert :          Invert the output
                DisableMathInvert :         De-Invert the output
                SetMathVDIV :               Configure VDIV of math channel
                SetMathVerticalPosition :   Set Math position
                SetFFTCenter :              Set FFT Center
                SetFTTDisplayMode :         Set FFT Display mode
                SetFFTVerticalPosition :    Set FFT Position
                SetFFTVerticalScale :       Set FFT Scale
                SetFFTHorizontalScale :     Set FFT Horizontal scale
                SetFFTVerticalUnit :        Set FFT Unit
                SetFFTWindow :              Set FFT Unit
    """

    def __init__(self, instr, baseclass, number=0):
        """
        Small overhide to add a variable to the class.
        """
        super().__init__(instr, baseclass, number)
        self.__VDIV__ = 0

    def DefineMathOperation(self, Equation):
        """
        pySDS [Maths][DefineMathOperation] : Configure math operation to do.

            Arguments :
                Equation : Human written operation

            Returns :
                self.GetAllErrors()
        """
        # First, check that the operation is valid
        # If a non authorized channel was passed -> Not replaced and trigger a -1
        Eq = (
            Equation.replace("C1", ";")
            .replace("C2", ";")
            .replace("C3", ";")
            .replace("C3", ";")
            .split(";")
        )
        for elem in Eq:
            if elem not in ["+", "-", "*", "/", "FFT", "INTG", "DIFF", "SQRT"]:
                return -1

        self.__instr__.write(f"DEFINE EQN,'{Equation}'")
        return self.__baseclass__.GetAllErrors()

    def EnableMathInvert(self):
        """
        pySDS [Math][EnableMathInvert] : Invert the trace of the math result

            Arguments :
                None

            Returns :
                self.GetAllErrors
        """
        self.__instr__.write("MATH:INVERTSET ON")
        return self.__baseclass__.GetAllErrors()

    def DisableMathInvert(self):
        """
        pySDS [Math][EnableMathInvert] : Disable inversion the trace of the math result

            Arguments :
                None

            Returns :
                self.GetAllErrors
        """
        self.__instr__.write("MATH:INVERTSET OFF")
        return self.__baseclass__.GetAllErrors()

    def SetMathVDIV(self, Unit):
        """
        pySDS [Math][SetMathVDIV] : Configure the vertical scale of the math channel

            Arguments :
                Unit : 500uV | 1mV | 2mV | 5mV | 10mV | 20mV | 50mV | 100mV | 200mV | 500mV | 1V | 2V | 5V | 10V | 20V | 50V | 100V

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid unit
        """

        if Unit not in [
            "500uV",
            "1mV",
            "2mV",
            "5mV",
            "10mV",
            "20mV",
            "50mV",
            "100mV",
            "200mV",
            "500mV",
            "1V",
            "2V",
            "5V",
            "10V",
            "20V",
            "50V",
            "100V",
        ]:
            return [1, -1]

        self.__instr__.write(f"MATH_VERT_DIV {Unit}")
        self.__VDIV__ = Unit
        return self.__baseclass__.GetAllErrors()

    def SetMathVerticalPosition(self, Offset):
        """
        pySDS [Math][SetMathVerticalPosition] : Configure the position of the math

            Arguments :
                Offset : Value in uV / mV / V to be offseted on the math signal

            Returns :
                self.GetAllErrors()
                or
                -1 : Out of range value (for this VDIV)
        """
        if (Offset < (-5 * self.__VDIV__)) or (Offset > (5 * self.__VDIV__)):
            return [1, -1]

        points = (Offset * 5) / self.__VDIV__
        self.__instr__.write(f"MATH_VERT_POS {points}")
        return self.__baseclass__.GetAllErrors()

    def SetFFTCenter(self, Value):
        """
        pySDS [Math][SetFFTCenter] : Configure the FFT Center point

        WARNING : It's not possible to check the value here

            Arguments :
                Value in Hz | kHz | MHz

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.write(f"FFTC {Value}")
        return self.__baseclass__.GetAllErrors()

    def SetFTTDisplayMode(self, Mode):
        """
        pySDS [Math][SetFFTDisplayMode] : Configure the mode of display of the FFT

            Arguments :
                Mode : ON | OFF | EXCLU

            Returns :
                self.GetAllErrors()
                or
                -1 : Value not allowed
        """
        if Mode not in ["ON", "OFF", "EXCLU"]:
            return [1, -1]

        self.__instr__.write(f"FFTF {Mode}")
        return self.__baseclass__.GetAllErrors()

    def SetFFTVerticalPosition(self, Offset):
        """
        pySDS [Math][SetFFTVerticalPosition] : Configure the VPOS of the FFT

            Arguments :
                Offset : Value in Volts

            Returns :
                self.GetAllErrors()
                or
                -1 : Out of range value
        """
        self.__instr__.write(f"FFTP {Offset}")
        return self.__baseclass__.GetAllErrors()

    def SetFFTVerticalScale(self, Scale):
        """
        pySDS [Math][SetFFTVerticalScale] : Set the FFT scale

        WARNING : Due to the option of two units, some values may be valid but not for this specific mode. VRMS is more restrictive.

            Arguments :
                Scale : Value to be used

            Returns :
                self.GetAllErrors()
                or
                -1 : Out of range value
        """
        if Scale not in [
            0.001,
            0.002,
            0.005,
            0.01,
            0.02,
            0.05,
            0.1,
            0.2,
            0.5,
            1,
            2,
            5,
            10,
            20,
        ]:
            return [1, -1]

        self.__instr__.write(f"FFTS {Scale}")
        return self.__baseclass__.GetAllErrors()

    def SetFFTHorizontalScale(self, Scale):
        """
        pySDS [Math][SetFFTHorizontalScale] : Set the horizontal division of the FFT trace

        WARNING : It's not possible to check the value here

            Arguments :
                Scale : in Hz | kHz | Mhz with unit

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.write(f"FFTT {Scale}")
        return self.__baseclass__.GetAllErrors()

    def SetFFTVerticalUnit(self, Unit):
        """
        pySDS [Math][SetFFTVerticalUnit] : Set the FFT unit. This function shall be called first when configuring FFT

            Arguments :
                Unit : VRMS | DBM | DBVRMS

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid Unit
        """
        if Unit not in ["VRMS", "DB", "DBVRMS"]:
            return [1, -1]

        self.__instr__.write(f"FFTU {Unit}")
        return self.__baseclass__.GetAllErrors()

    def SetFFTWindow(self, Window):
        """
        pySDS [Math][SetFFTWindow] : Configure the FFT Window to be used

            Arguments :
                Window : RECT | BLAC | HANN | HAMM | FLATTOP

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid Window

        Doc from siglent :
            RECT — Rectangle is useful for transient signals, and signals where there are an integral number of cycles in the time record.
            BLAC — Blackman reduces time resolution compared to the rectangular window, but it improves the capacity to detect smaller impulses due to lower secondary lobes (provides minimal spectral leakage).
            HANN — Hanning is useful for frequency resolution and general purpose use. It is good for resolving two frequencies that are close together, or for making frequency measurements.
            HAMM — Hamming.
            FLATTOP — Flattop is the best for making accurate amplitude measurements of frequency peaks
        """
        if Window not in ["RECT", "BLAC", "HANN", "HAMM", "FLATTOP"]:
            return [1, -1]

        self.__instr__.write(f"FFTW {Window}")
        return self.__baseclass__.GetAllErrors()



# ============================================================================================================
# Waveform.py
# lheywang on 21/12/2024
#
# Base file for the waveform class
#
# ============================================================================================================
from ..BaseOptionnalClass import SiglentBase
from ..channel import SiglentChannel


class SiglentPassFail(SiglentBase):
    """
    pySDS [Files][SiglentPassFail] :    Class herited from SiglentBase.
                                        Store all command related the control of automated tests

        Attributes :
            Herited from SiglentBase

        Methods :
            Private (0) :
                None

            Public (14):
                ClearTests :                    Clear tests results
                EnableBuzzerOnFail :            Enable buzzer
                DisableBuzzerOnFail :           Disable buzzer
                CreateRule :                    Create new rule
                GetFramesResults :              Get results
                EnableInformationDisplay :      Show infos on screen
                DisableInformationDisplay :     Hide infos on screen
                EnablePassFailMode :            Enable mode
                DisablePassFailMode :           Disable mode
                EnableStopOnFail :              Enable stop on fail
                DisableStopOnFail :             Disable stop on fail
                Runtest :                       Run the test
                Stoptest :                      Stop the test
                SetSource :                     Set source
                SetTolerances :                 Configure tolerances
    """

    def ClearTests(self):
        """
        pySDS [PassFail][ClearTests] : Clear the results of the test

            Arguments :
                None

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.write("PACL")
        return self.__baseclass__.GetAllErrors()

    def EnableBuzzerOnFail(self):
        """
        pySDS [PassFail][EnableBuzzerOnFail] : Enable the buzzer when a fail is detected

        WARNING : The documentation is not clear about the real usage of this function. Use at your own risks !

            Arguments :
                None

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.write("PFBF ON")
        return self.__baseclass__.GetAllErrors()

    def DisableBuzzerOnFail(self):
        """
        pySDS [PassFail][DisableBuzzerOnFail] : Disable the buzzer when a fail is detected

        WARNING : The documentation is not clear about the real usage of this function. Use at your own risks !

            Arguments :
                None

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.write("PFBF OFF")
        return self.__baseclass__.GetAllErrors()

    def CreateRule(self):
        """
        pySDS [PassFail][CreateRule] : Create a pass fail test around the selected channel.

            Arguments :
                None

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.write("PFCM")
        return self.__baseclass__.GetAllErrors()

    def GetFramesResults(self):
        """
        pySDS [PassFail][GetFramesResults] : Return the number of frames that passed the test, failed and total

            Arguments :
                None

            Returns :
                List of values : Failed, Passed, Total
        """
        ret = self.__instr__.query("PFDD?").strip().split(" ")[-1].split(",")
        return [int(ret[1]), int(ret[3]), int(ret[5])]

    def EnableInformationDisplay(self):
        """
        pySDS [PassFail][EnableInformationDisplay] : Enable on the screen the information panel

            Arguments :
                None

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.write("PFDS ON")
        return self.__baseclass__.GetAllErrors()

    def DisableInformationDisplay(self):
        """
        pySDS [PassFail][DisableInformationDisplay] : Disable on the screen the information panel

            Arguments :
                None

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.write("PFDS OFF")
        return self.__baseclass__.GetAllErrors()

    def EnablePassFailMode(self):
        """
        pySDS [PassFail][EnablePassFailMode] : Enable the pass/fail mode on the scope

            Arguments :
                None

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.write("PFEN ON")
        return self.__baseclass__.GetAllErrors()

    def DisablePassFailMode(self):
        """
        pySDS [PassFail][DisablePassFailMode] : Disable the pass/fail mode on the scope

            Arguments :
                None

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.write("PFEN OFF")
        return self.__baseclass__.GetAllErrors()

    def EnableStopOnFail(self):
        """
        pySDS [PassFail][EnableStopOnFail] : Stop the scope once the first fail has been detected

            Arguments :
                None

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.write("PFFS ON")
        return self.__baseclass__.GetAllErrors()

    def DisableStopOnFail(self):
        """
        pySDS [PassFail][DisableStopOnFail] : Do not stop the scope once the first fail has been detected

            Arguments :
                None

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.write("PFFS OFF")
        return self.__baseclass__.GetAllErrors()

    def Runtest(self):
        """
        pySDS [PassFail][RunTest] : Launch the test execution

            Arguments :
                None

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.write("PFOP ON")
        return self.__baseclass__.GetAllErrors()

    def Stoptest(self):
        """
        pySDS [PassFail][Stoptest] : Stop the test execution

            Arguments :
                None

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.write("PFOP OFF")
        return self.__baseclass__.GetAllErrors()

    def SetSource(self, Channel: SiglentChannel):
        """
        pySDS [PassFail][SetSource] : Set the source for the selftest

            Arguments :
                Channel : SiglentChannel class to be used

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid channel descriptor
        """
        if type(Channel) is not SiglentChannel:
            return [1, -1]

        self.__instr__.write(f"PFSC {Channel.__channel__}")
        return self.__baseclass__.GetAllErrors()

    def SetTolerances(self, X, Y):
        """
        pySDS [PassFail][SetTolerances] : Set the X and Y tolerances for this source (Tolerances are expressed as on screen DIV)

            Arguments :
                X : X tolerance (between 0.04 and 4)
                Y : Y tolerance (between 0.04 and 4)

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid X tolerance
                -2 : Invalid Y tolerance
        """
        if X < 0.04 or X > 4:
            return [1, -1]
        if Y < 0.04 or Y > 4:
            return [1, -2]

        self.__instr__.write(
            f"PFST XMASK,{round(X / 0.04) * 0.04:.2f},YMASK{round(Y / 0.04) * 0.04:.2f}"
        )
        return self.__baseclass__.GetAllErrors()



# ============================================================================================================
# BaseOptionnalClass.py
# lheywang on 17/12/2024
#
# Base class, without real usage except to serve as common base for all options
#
# ============================================================================================================


class SiglentBase:
    """
    pySDS [SiglentBase] : Standard class for all of SCPI subsystems. May be derivated into more specific options, if needed

        Attributes :
            Private (2):
                __instr__ :     Handle to the pyvisa class linked to the device
                __baseclass__   Reference to the base class to get standard functions
            Public (0):
                None

        Methods :
            Private (0):
                None

            Public (0):
                None

        Parents :
            None

        Subclass :
            All of the SCPI substystem class (Acquisition, WGEN...)
            Channel modify __init__.
    """

    def __init__(self, instr, baseclass, number=0):
        self.__instr__ = instr
        self.__baseclass__ = baseclass



ref = input("Name of the scope without SDS ?     ")
channel_nb = int(input("Number of channels      "))

with open(ref + ".toml", "w+") as f:
    f.write("[Infos]\n")
    f.write('Brand = "Siglent"\n')
    f.write(f'Device = "SDS{ref}"\n')
    f.write("\n")
    f.write("[Specs]\n")
    f.write(f"Channels = {channel_nb}\n")

    if channel_nb == 2:
        f.write('ChannelColor = ["Red", "Yellow"]\n')
    elif channel_nb == 4:
        f.write('ChannelColor = ["Red", "Yellow", "Blue", "Green"]\n')
    elif channel_nb == 8:
        f.write(
            'ChannelColor = ["Yellow", "Pink", "Light Blue", "Green", "Violet", "Blue", "Red", "Orange"]\n'
        )

    f.write("LegacyFunctions = []\n")
    f.write("BlacklistedFUnctions = []\n")
    f.write("impedance = [1000000]")
    f.write("\n")

    f.write("[Generator]\n")
    f.write("Enabled = false\n")
    f.write("Channel = 1\n")
    f.write("\n")

    f.write("[MixedSignal]\n")
    f.write("Enabled = false\n")
    f.write("Channel = 16\n")
    f.write("\n")

    f.write("[PowerAnalyser]\n")
    f.write("Enabled = false\n")
    f.write("\n")

print("Done !")



# ============================================================================================================
# Acquisition.py
# lheywang on 19/12/2024
#
# Base file for the acquisition class
#
# ============================================================================================================
from ..BaseOptionnalClass import SiglentBase


class SiglentAcquisition(SiglentBase):
    """
    pySDS [Acquision][SiglentAcquisition] :   C lass herited from SiglentBase.
                                                Store all command related to the control of acquision
        Attributes :
            Herited from SiglentBase

        Methods :
            Private (0) :
                None

            Public (13):
                Arm :                       Prepare the device to be ready to trigger
                Stop :                      Stop the device to be ready to trigger
                ConfigureAquireMethod :     Configure the way of acquiring data
                SetAverageCount :           Configure the average number of sample
                GetAverageCount :           Get the number of average samples
                GetMemorySize :             Get the size in sample of the memory used
                SetMemorySize :             Configure the size in sample of the memory
                GetAcquisitionStatus :      Return the acquision status
                GetSampleRate :             Return the used sample rate (function of time resolution and channel)
                GetSampleNumber :           Return the number of sample stored
                SetInterpolationMethod :    Configure the interpolation method to be used (only on display, inter points)
                EnableXYMode :              Enable the XY mode
                DisableXYMode :             Disable the XY mode
    """

    def Arm(self):
        """
        pySDS [Acquisition][Arm] : Place the device to be ready to acquire a waveform once a triggering condition has been validated

            Arguments :
                None

            Returns :
                self.GetDeviceStatus()[13] which take 1 if trigger is ready, 0 otherwise
        """
        self.__instr__.write("ARM")
        return self.__baseclass__.GetDeviceStatus()[13]

    def Stop(self):
        """
        pySDS [Acquisition][Stop] : Stop the device to be ready to acquire a waveform.

            Arguments :
                None

            Returns :
                self.GetDeviceStatus()[13] which take 1 if trigger is cancelled, 0 otherwise
        """
        self.__instr__.write("STOP")
        return not self.__baseclass__.GetDeviceStatus[13]

    def ConfigureAquireMethod(self, Method: str, AverageNumber: int = 1):
        """
        pySDS [Acquisition][ConfigureAquireMethod] : Configure the way the device handle data acquisition

            Arguments :
                Method : SAMPLING | PEAK_DETECT | AVERAGE | HIGH_RES
                AverageNumber : Number of sample used to compute an average point

            Returns :
                0 | -1 : The device responded with the same settings or differents one.
        """

        if Method not in ["SAMPLING", "PEAK_DETECT", "AVERAGE", "HIGH_RES"]:
            return -2

        self.__instr__.write(f"ACQW {Method},{AverageNumber}")
        return self.__baseclass__.GetAllErrors()

    def SetAverageCount(self, AverageNumber):
        """
        pySDS [Acquisition][SetAverageCount] : Configure the number of sampled used per average

            Arguments :
                AverageNumber : Number of sample used to compute an average point

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.write(f"AVGA {AverageNumber}")

        return self.__baseclass__.GetAllErrors()

    def GetAverageCount(self):
        """
        pySDS [Acquisition][GetAverageCount] : Return the number of sample used for averaging

            Arguments :
                None

            Returns :
                Integer : Number of samples
        """
        return int(self.__instr__.write("AVGA?").strip().split(" ")[-1])

    def GetMemorySize(self):
        """
        PySDS [Acquisition][GetMemorySize] : Return the number in millions of samples that can be stored into the memory

        WARNING : The value is expressed in number of samples, and not in bytes !

            Arguments :
                None

            Returns :
                Integer : The number of **MILLIONS** of sample that can be stored
        """
        Ret = self.__instr__.query("MSIZ?")
        return int(Ret.strip().split(" ")[-1][:-1])

    def SetMemorySize(self, value: int):
        """
        PySDS [Acquisition][SetMemorySize] : Set the memory size for the samples of the scope.

        WARNING : The value is expressed in number of samples, and not in bytes !

            Arguments :
                The value in **MILLIONS** to the used.

            Returns :
                self.GetAllErrors() returns (List of errors)
        """
        self.__instr__.write(f"MSIZ {value}M")
        return self.__baseclass__.GetAllErrors()

    def GetAcquisitionStatus(self):
        """
        PySDS [Acquisition][GetAcquisitionStatus] : Return the acquisition status of the device

            Arguments :
                None

            Returns :
                String : Device response
        """

        return self.__instr__.query("SAST?").strip().split(" ")[-1]

    def GetSampleRate(self):
        """
        PySDS [Acquisition][GetSampleRate] : Return the acquisition sample rate that is actually used

            Arguments :
                None

            Returns :
                String : Device response
        """
        return self.__instr__.query("SARA?").strip().split(" ")[-1]

    def GetSampleNumber(self):
        """
        PySDS [Acquisition][GetSampleNumber] : Return the acquisition number of points captured

            Arguments :
                None

            Returns :
                String : Device response
        """
        return self.__instr__.query("SANU?").strip().split(" ")[-1]

    def SetInterpolationMethod(self, Method):
        """
        PySDS [Acquisition][SetInterpolationMethod] :   Configure the interpolation method to be used

            Arguments :
                Method : ON | OFF (sine interpolation or linear interpolation)

            Returns :
                self.GetAllErrors()
        """
        if Method not in ["ON", "OFF"]:
            return -2

        self.__instr__.write(f"SXSA {Method}")
        return self.__baseclass__.GetAllErrors()

    def EnableXYMode(self):
        """
        PySDS [Acquisition][EnableXYMode] :   Enable the XY mode

            Arguments :
                None

            Returns :
                self.GetAllErrors()
        """

        self.__instr__.write("XYDS ON")
        return self.__baseclass__.GetAllErrors()

    def DisableXYMode(self):
        """
        PySDS [Acquisition][DisableXYMode] :    Disable the XY mode

            Arguments :
                None

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.write("XYDS OFF")
        return self.__baseclass__.GetAllErrors()
